//
//  UITableView+Split.m
//  SplitTableView
//
//  Created by qiyu on 10/23/12.
//  Copyright (c) 2013 qiyu. All rights reserved.
//

#import "UITableView+Split.h"
#import <QuartzCore/QuartzCore.h>

#define kSplitTag 11990
#define kTopTag 11991
#define kBottomTag 11992
#define kTriangleTag 11993
#define kTopAlphaTag 11994
#define kBottomAlphaTag 11995


@implementation UITableView (Split)
- (void)splitTableViewAtRowForIndexPath:(NSIndexPath*)indexPath withView:(UIView*)splitterView
{
    if([self viewWithTag:kSplitTag]){
        return;
    }
    
    UITableViewCell *cell = [self cellForRowAtIndexPath:indexPath];
    if(!cell){
        return;
    }
    
    CGFloat w = self.bounds.size.width;
    splitterView.tag = kSplitTag;
    CGRect cellFrame = [cell convertRect:cell.bounds toView:self.superview];
    CGFloat y = CGRectGetMaxY(cellFrame);
    if(y > CGRectGetMaxY(self.frame)){
        NSIndexPath *indexPath = [self indexPathForCell:cell];
        [self scrollToRowAtIndexPath:indexPath atScrollPosition:UITableViewScrollPositionBottom animated:NO];
        cellFrame = [cell convertRect:cell.bounds toView:self.superview];
        y = CGRectGetMaxY(cellFrame);
    }
    
    UIGraphicsBeginImageContextWithOptions(self.frame.size, self.opaque, [[UIScreen mainScreen] scale]);
    CGContextRef ctx = UIGraphicsGetCurrentContext();
    CGContextTranslateCTM(ctx, 0, -self.contentOffset.y);
    [self.layer renderInContext:UIGraphicsGetCurrentContext()];
    UIImage *snapshotImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    
    CGFloat topPartMargin = y - self.frame.origin.y;
    CGFloat bottomPartMargin = self.frame.size.height - topPartMargin;
    
    //Top View
    UIView *topPartView = [[[UIView alloc] initWithFrame:CGRectMake(0, self.frame.origin.y, w, topPartMargin)] autorelease];
    topPartView.backgroundColor = [UIColor blueColor];
    topPartView.clipsToBounds = YES;
    topPartView.tag = kTopTag;
    
    UIImageView *topPartImageView = [[[UIImageView alloc] initWithImage:snapshotImage] autorelease];
    [topPartView addSubview:topPartImageView];
    
    UIImageView *arrowView = [[[UIImageView alloc] initWithImage:[UIImage imageNamed:@"triangle"]] autorelease];
    CGRect arrowFrame = arrowView.frame;
    arrowFrame.origin.x = 0;
    arrowFrame.origin.y = topPartView.frame.size.height - arrowFrame.size.height;
    arrowView.frame = arrowFrame;
    arrowView.alpha = 0.0;
    arrowView.tag = kTriangleTag;
    [topPartView addSubview:arrowView];
    
    UIView *topPartAlphaView = [[[UIView alloc] initWithFrame:CGRectMake(0, 0, w, topPartMargin - cell.bounds.size.height)] autorelease];
    topPartAlphaView.tag = kTopAlphaTag;
    topPartAlphaView.alpha = 0.0;
    topPartAlphaView.backgroundColor = [UIColor blackColor];
    [topPartView addSubview:topPartAlphaView];
    
    //Bottom View
    UIView *bottomPartView = [[[UIView alloc] initWithFrame:CGRectMake(0, y, w, bottomPartMargin)] autorelease];
    bottomPartView.clipsToBounds = YES;
    bottomPartView.tag = kBottomTag;
    
    UIImageView *bottomPartImageView = [[[UIImageView alloc] initWithImage:snapshotImage] autorelease];
    CGRect bottomPartImageFrame = bottomPartImageView.frame;
    bottomPartImageFrame.origin.y = -topPartMargin;
    bottomPartImageView.frame = bottomPartImageFrame;
    [bottomPartView addSubview:bottomPartImageView];
    
    UIView *bottomPartAlphaView = [[[UIView alloc] initWithFrame:bottomPartView.bounds] autorelease];
    bottomPartAlphaView.tag = kBottomAlphaTag;
    bottomPartAlphaView.alpha = 0.0;
    bottomPartAlphaView.backgroundColor = [UIColor blackColor];
    [bottomPartView addSubview:bottomPartAlphaView];
    
    UITapGestureRecognizer *topPartRecognizer = [[[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(onTapTableViewSnapShotView:)] autorelease];
    [topPartView addGestureRecognizer:topPartRecognizer];
    
    UITapGestureRecognizer *bottomPartRecognizer = [[[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(onTapTableViewSnapShotView:)] autorelease];
    [bottomPartView addGestureRecognizer:bottomPartRecognizer];
    
    CGRect actionFrame = splitterView.frame;
    [self.superview insertSubview:splitterView aboveSubview:self];
    [self.superview insertSubview:topPartView aboveSubview:splitterView];
    [self.superview insertSubview:bottomPartView aboveSubview:splitterView];
    if(bottomPartMargin >= splitterView.bounds.size.height){
        actionFrame.origin.y = y;
        splitterView.frame = actionFrame;
        CGRect bottomPartFrame = bottomPartView.frame;
        bottomPartFrame.origin.y = CGRectGetMaxY(actionFrame);
        [UIView animateWithDuration:0.25 animations:^{
            topPartAlphaView.alpha = 0.2;
            bottomPartAlphaView.alpha = 0.2;
            arrowView.alpha = 1.0;
            bottomPartView.frame = bottomPartFrame;
        }];
    }
    else {
        actionFrame.origin.y = y - actionFrame.size.height;
        splitterView.frame = actionFrame;
        CGRect topPartFrame = topPartView.frame;
        topPartFrame.origin.y = CGRectGetMinY(actionFrame) - topPartFrame.size.height;
        [UIView animateWithDuration:0.25 animations:^{
            topPartAlphaView.alpha = 0.2;
            bottomPartAlphaView.alpha = 0.2;
            arrowView.alpha = 1.0;
            topPartView.frame = topPartFrame;
        }];
    }
}

- (void)onTapTableViewSnapShotView:(id)sender
{
    [self restoreFromSplittedStateWithCompletion:nil];
}


- (BOOL)splitted
{
    return [self.superview viewWithTag:kTopTag] != nil;
}

- (void)restoreFromSplittedState
{
    UIView *topPartView = [self.superview viewWithTag:kTopTag];
    UIView *bottomPartView = [self.superview viewWithTag:kBottomTag];
    UIView *splitterView = [self.superview viewWithTag:kSplitTag];
    
    [topPartView removeFromSuperview];
    [bottomPartView removeFromSuperview];
    [splitterView removeFromSuperview];
}

- (void)restoreFromSplittedStateWithCompletion:(void (^)(void))completion
{
    UIView *topPartView = [self.superview viewWithTag:kTopTag];
    UIView *bottomPartView = [self.superview viewWithTag:kBottomTag];
    UIView *splitterView = [self.superview viewWithTag:kSplitTag];
    UIView *arrowView = [topPartView viewWithTag:kTriangleTag];
    UIView *topPartAlphaView = [topPartView viewWithTag:kTopAlphaTag];
    UIView *bottomPartAlphaView = [bottomPartView viewWithTag:kBottomAlphaTag];
    [UIView animateWithDuration:0.25 animations:^{
        if(topPartView.frame.origin.y < self.frame.origin.y){
            CGRect f = topPartView.frame;
            f.origin.y = self.frame.origin.y;
            topPartView.frame = f;
        }
        else {
            CGRect f = bottomPartView.frame;
            f.origin.y = CGRectGetMaxY(topPartView.frame);
            bottomPartView.frame = f;
        }
        arrowView.alpha = 1.0;
        topPartAlphaView.alpha = 0.0;
        bottomPartAlphaView.alpha = 0.0;
    } completion:^(BOOL finished) {
        [splitterView removeFromSuperview];
        [topPartView removeFromSuperview];
        [bottomPartView removeFromSuperview];
        if(completion){
            completion();
        }
    }];
}

@end
